@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -48,15 +48,44 @@ extern int		server_num, process_num;
 					ZBX_DATASENDER_AUTOREGISTRATION | ZBX_DATASENDER_TASKS |	\
 					ZBX_DATASENDER_TASKS_RECV)
 
+/******************************************************************************
+ *                                                                            *
+ * Function: get_hist_upload_state                                            *
+ *                                                                            *
+ * Purpose: Get current history upload state (disabled/enabled)               *
+ *                                                                            *
+ * Parameters: buffer - [IN] the contents of a packet (JSON)                  *
+ *                                                                            *
+ * Return value: SUCCEED - processed successfully                             *
+ *               FAIL - an error occurred                                     *
+ *                                                                            *
+ ******************************************************************************/
+static void	get_hist_upload_state(const char *buffer, int *state)
+{
+	struct zbx_json_parse	jp;
+	char			value[MAX_STRING_LEN];
+
+	if (NULL == buffer || '\0' == *buffer || SUCCEED != zbx_json_open(buffer, &jp))
+		return;
+
+	if (SUCCEED == zbx_json_value_by_name(&jp, ZBX_PROTO_TAG_PROXY_UPLOAD, value, sizeof(value), NULL))
+	{
+		if (0 == strcmp(value, ZBX_PROTO_VALUE_PROXY_UPLOAD_ENABLED))
+			*state = ZBX_PROXY_UPLOAD_ENABLED;
+		else if (0 == strcmp(value, ZBX_PROTO_VALUE_PROXY_UPLOAD_DISABLED))
+			*state = ZBX_PROXY_UPLOAD_DISABLED;
+	}
+}
+
 /******************************************************************************
  *                                                                            *
  * Function: proxy_data_sender                                                *
  *                                                                            *
- * Purpose: collects host availability, history, discovery, auto registration *
+ * Purpose: collects host availability, history, discovery, autoregistration  *
  *          data and sends 'proxy data' request                               *
  *                                                                            *
  ******************************************************************************/
-static int	proxy_data_sender(int *more, int now)
+static int	proxy_data_sender(int *more, int now, int *hist_upload_state)
 {
 	static int		data_timestamp = 0, task_timestamp = 0, upload_state = SUCCEED;
 
@@ -64,7 +93,7 @@ static int	proxy_data_sender(int *more, int now)
 	struct zbx_json		j;
 	struct zbx_json_parse	jp, jp_tasks;
 	int			availability_ts, history_records = 0, discovery_records = 0,
-				areg_records = 0, more_history = 0, more_discovery = 0, more_areg = 0;
+				areg_records = 0, more_history = 0, more_discovery = 0, more_areg = 0, proxy_delay;
 	zbx_uint64_t		history_lastid = 0, discovery_lastid = 0, areg_lastid = 0, flags = 0;
 	zbx_timespec_t		ts;
 	char			*error = NULL;
@@ -79,7 +108,8 @@ static int	proxy_data_sender(int *more, int now)
 	zbx_json_addstring(&j, ZBX_PROTO_TAG_HOST, CONFIG_HOSTNAME, ZBX_JSON_TYPE_STRING);
 	zbx_json_addstring(&j, ZBX_PROTO_TAG_SESSION, zbx_dc_get_session_token(), ZBX_JSON_TYPE_STRING);
 
-	if (SUCCEED == upload_state && CONFIG_PROXYDATA_FREQUENCY <= now - data_timestamp)
+	if (SUCCEED == upload_state && CONFIG_PROXYDATA_FREQUENCY <= now - data_timestamp &&
+			ZBX_PROXY_UPLOAD_DISABLED != *hist_upload_state)
 	{
 		if (SUCCEED == get_host_availability_data(&j, &availability_ts))
 			flags |= ZBX_DATASENDER_AVAILABILITY;
@@ -142,11 +172,20 @@ static int	proxy_data_sender(int *more, int now)
 		zbx_json_adduint64(&j, ZBX_PROTO_TAG_CLOCK, ts.sec);
 		zbx_json_adduint64(&j, ZBX_PROTO_TAG_NS, ts.ns);
 
-		if (SUCCEED != (upload_state = put_data_to_server(&sock, &j, &error)))
+		if (0 != (flags & ZBX_DATASENDER_HISTORY) && 0 != (proxy_delay = proxy_get_delay(history_lastid)))
+			zbx_json_adduint64(&j, ZBX_PROTO_TAG_PROXY_DELAY, proxy_delay);
+
+		upload_state = put_data_to_server(&sock, &j, &error);
+		get_hist_upload_state(sock.buffer, hist_upload_state);
+
+		if (SUCCEED != upload_state)
 		{
 			*more = ZBX_PROXY_DATA_DONE;
-			zabbix_log(LOG_LEVEL_WARNING, "cannot send proxy data to server at \"%s\": %s",
-					sock.peer, error);
+			if (ZBX_PROXY_UPLOAD_DISABLED != *hist_upload_state)
+			{
+				zabbix_log(LOG_LEVEL_WARNING, "cannot send proxy data to server at \"%s\": %s",
+						sock.peer, error);
+			}
 			zbx_free(error);
 		}
 		else
@@ -212,7 +251,7 @@ clean:
  ******************************************************************************/
 ZBX_THREAD_ENTRY(datasender_thread, args)
 {
-	int		records = 0, more;
+	int		records = 0, hist_upload_state = ZBX_PROXY_UPLOAD_ENABLED, more;
 	double		time_start, time_diff = 0.0, time_now;
 
 	process_type = ((zbx_thread_args_t *)args)->process_type;
@@ -224,7 +263,7 @@ ZBX_THREAD_ENTRY(datasender_thread, args)
 
 	update_selfmon_counter(ZBX_PROCESS_STATE_BUSY);
 
-#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+#if defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
 	zbx_tls_init_child();
 #endif
 	zbx_setproctitle("%s [connecting to the database]", get_process_type_string(process_type));
@@ -244,7 +283,7 @@ ZBX_THREAD_ENTRY(datasender_thread, args)
 
 		do
 		{
-			records += proxy_data_sender(&more, (int)time_now);
+			records += proxy_data_sender(&more, (int)time_now, &hist_upload_state);
 
 			time_now = zbx_time();
 			time_diff = time_now - time_start;
