@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -693,6 +693,8 @@ static void	vmware_get_events(const zbx_vector_ptr_t *events, zbx_uint64_t event
 
 			zbx_vector_ptr_append(add_results, add_result);
 		}
+		else
+			zbx_free(add_result);
 	}
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s(): events:%d", __func__, add_results->values_num);
@@ -722,7 +724,7 @@ int	check_vcenter_eventlog(AGENT_REQUEST *request, const DC_ITEM *item, AGENT_RE
 	}
 	else if (0 == strcmp(skip, "skip"))
 	{
-		skip_old = 1;
+		skip_old = (0 == request->lastlogsize ? 1 : 0);
 	}
 	else
 	{
@@ -735,12 +737,19 @@ int	check_vcenter_eventlog(AGENT_REQUEST *request, const DC_ITEM *item, AGENT_RE
 	if (NULL == (service = get_vmware_service(url, item->username, item->password, result, &ret)))
 		goto unlock;
 
-	if (ZBX_VMWARE_EVENT_KEY_UNINITIALIZED == service->eventlog.last_key)
+	if (ZBX_VMWARE_EVENT_KEY_UNINITIALIZED == service->eventlog.last_key ||
+			(0 != skip_old && 0 != service->eventlog.last_key ))
 	{
+		/* this may happen if recreate item vmware.eventlog for the same service URL */
 		service->eventlog.last_key = request->lastlogsize;
 		service->eventlog.skip_old = skip_old;
 	}
-	else if (request->lastlogsize < service->eventlog.last_key)
+	else if (0 != service->eventlog.oom)
+	{
+		SET_MSG_RESULT(result, zbx_strdup(NULL, "Not enough shared memory to store VMware events."));
+		goto unlock;
+	}
+	else if (request->lastlogsize < service->eventlog.last_key && 0 != request->lastlogsize)
 	{
 		/* this may happen if there are multiple vmware.eventlog items for the same service URL or item has  */
 		/* been polled, but values got stuck in history cache and item's lastlogsize hasn't been updated yet */
@@ -1502,13 +1511,14 @@ out:
 }
 
 static int	check_vcenter_hv_datastore_latency(AGENT_REQUEST *request, const char *username, const char *password,
-		const char *perfcounter, AGENT_RESULT *result)
+		const char *perfcounter, zbx_uint64_t access_filter, AGENT_RESULT *result)
 {
 	char			*url, *mode, *uuid, *name;
 	zbx_vmware_service_t	*service;
 	zbx_vmware_hv_t		*hv;
 	zbx_vmware_datastore_t	*datastore;
-	int			ret = SYSINFO_RET_FAIL;
+	int			i, ret = SYSINFO_RET_FAIL;
+	zbx_str_uint64_pair_t	uuid_cmp = {.value = 0};
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() perfcounter:%s", __func__, perfcounter);
 
@@ -1561,6 +1571,28 @@ static int	check_vcenter_hv_datastore_latency(AGENT_REQUEST *request, const char
 		goto unlock;
 	}
 
+	uuid_cmp.name = hv->uuid;
+
+	if (FAIL == (i = zbx_vector_str_uint64_pair_bsearch(&datastore->hv_uuids_access, uuid_cmp,
+			zbx_str_uint64_pair_name_compare)))
+	{
+		SET_MSG_RESULT(result, zbx_dsprintf(NULL, "Unknown hypervisor \"%s\" for datastore \"%s\".",
+				hv->props[ZBX_VMWARE_HVPROP_NAME], datastore->name));
+		goto unlock;
+	}
+
+	if (access_filter != (datastore->hv_uuids_access.values[i].value & access_filter))
+	{
+		zbx_uint64_t	mi = datastore->hv_uuids_access.values[i].value;
+
+		SET_MSG_RESULT(result, zbx_dsprintf(NULL, "Datastore is not available for hypervisor: %s",
+				0 == (ZBX_VMWARE_DS_MOUNTED & mi) ? "unmounted" : (
+				0 == (ZBX_VMWARE_DS_ACCESSIBLE & mi) ? "inaccessible" : (
+				ZBX_VMWARE_DS_READ == (ZBX_VMWARE_DS_READWRITE & mi)? "readOnly" :
+				"unknown"))));
+		goto unlock;
+	}
+
 	ret = vmware_service_get_counter_value_by_path(service, "HostSystem", hv->id, perfcounter, datastore->uuid, 1,
 			result);
 unlock:
@@ -1575,14 +1607,14 @@ int	check_vcenter_hv_datastore_read(AGENT_REQUEST *request, const char *username
 		AGENT_RESULT *result)
 {
 	return check_vcenter_hv_datastore_latency(request, username, password, "datastore/totalReadLatency[average]",
-			result);
+			ZBX_VMWARE_DS_READ_FILTER, result);
 }
 
 int	check_vcenter_hv_datastore_write(AGENT_REQUEST *request, const char *username, const char *password,
 		AGENT_RESULT *result)
 {
 	return check_vcenter_hv_datastore_latency(request, username, password, "datastore/totalWriteLatency[average]",
-			result);
+			ZBX_VMWARE_DS_WRITE_FILTER, result);
 }
 
 static int	check_vcenter_hv_datastore_size_vsphere(int mode, const zbx_vmware_datastore_t *datastore,
@@ -1670,6 +1702,7 @@ static int	check_vcenter_ds_size(const char *url, const char *hv_uuid, const cha
 	zbx_vmware_datastore_t	*datastore = NULL;
 	zbx_uint64_t		disk_used, disk_provisioned, disk_capacity;
 	unsigned int		flags;
+	zbx_str_uint64_pair_t	uuid_cmp = {.name = (char *)hv_uuid, .value = 0};
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
@@ -1687,7 +1720,8 @@ static int	check_vcenter_ds_size(const char *url, const char *hv_uuid, const cha
 	}
 
 	if (NULL != hv_uuid &&
-			FAIL == zbx_vector_str_bsearch(&datastore->hv_uuids, hv_uuid, ZBX_DEFAULT_STR_COMPARE_FUNC))
+			FAIL == zbx_vector_str_uint64_pair_bsearch(&datastore->hv_uuids_access, uuid_cmp,
+			zbx_str_uint64_pair_name_compare))
 	{
 		SET_MSG_RESULT(result, zbx_dsprintf(NULL, "Hypervisor '%s' not found on this datastore.", hv_uuid));
 		goto unlock;
@@ -1956,9 +1990,9 @@ int	check_vcenter_datastore_hv_list(AGENT_REQUEST *request, const char *username
 		goto unlock;
 	}
 
-	for (i=0; i < datastore->hv_uuids.values_num; i++)
+	for (i=0; i < datastore->hv_uuids_access.values_num; i++)
 	{
-		if (NULL == (hv = hv_get(&service->data->hvs, datastore->hv_uuids.values[i])))
+		if (NULL == (hv = hv_get(&service->data->hvs, datastore->hv_uuids_access.values[i].name)))
 		{
 			SET_MSG_RESULT(result, zbx_strdup(NULL, "Unknown hypervisor uuid."));
 			zbx_free(hv_list);
@@ -1966,7 +2000,7 @@ int	check_vcenter_datastore_hv_list(AGENT_REQUEST *request, const char *username
 		}
 
 		if (NULL == (hv_name = hv->props[ZBX_VMWARE_HVPROP_NAME]))
-			hv_name = datastore->hv_uuids.values[i];
+			hv_name = datastore->hv_uuids_access.values[i].name;
 
 		hv_list = zbx_strdcatf(hv_list, "%s\n", hv_name);
 	}
@@ -2065,13 +2099,13 @@ out:
 }
 
 static int	check_vcenter_datastore_latency(AGENT_REQUEST *request, const char *username, const char *password,
-		const char *perfcounter, AGENT_RESULT *result)
+		const char *perfcounter, zbx_uint64_t access_filter, AGENT_RESULT *result)
 {
 	char			*url, *mode, *name;
 	zbx_vmware_service_t	*service;
 	zbx_vmware_hv_t		*hv;
 	zbx_vmware_datastore_t	*datastore;
-	int			i, ret = SYSINFO_RET_FAIL, count = 0;
+	int			i, ret = SYSINFO_RET_FAIL, count = 0, ds_count = 0;
 	zbx_uint64_t		latency = 0, counterid;
 	unsigned char		is_maxlatency = 0;
 
@@ -2121,9 +2155,22 @@ static int	check_vcenter_datastore_latency(AGENT_REQUEST *request, const char *u
 		goto unlock;
 	}
 
-	for (i = 0; i < datastore->hv_uuids.values_num; i++)
+	for (i = 0; i < datastore->hv_uuids_access.values_num; i++)
 	{
-		if (NULL == (hv = hv_get(&service->data->hvs, datastore->hv_uuids.values[i])))
+		if (access_filter != (datastore->hv_uuids_access.values[i].value & access_filter))
+		{
+			zbx_uint64_t	mi = datastore->hv_uuids_access.values[i].value;
+
+			zabbix_log(LOG_LEVEL_DEBUG, "Datastore %s is not available for hypervisor %s: %s",
+					datastore->name, datastore->hv_uuids_access.values[i].name,
+					0 == (ZBX_VMWARE_DS_MOUNTED & mi) ? "unmounted" : (
+					0 == (ZBX_VMWARE_DS_ACCESSIBLE & mi) ? "inaccessible" : (
+					ZBX_VMWARE_DS_READ == (ZBX_VMWARE_DS_READWRITE & mi)? "readOnly" :
+					"unknown")));
+			continue;
+		}
+
+		if (NULL == (hv = hv_get(&service->data->hvs, datastore->hv_uuids_access.values[i].name)))
 		{
 			SET_MSG_RESULT(result, zbx_strdup(NULL, "Unknown hypervisor uuid."));
 			goto unlock;
@@ -2135,6 +2182,8 @@ static int	check_vcenter_datastore_latency(AGENT_REQUEST *request, const char *u
 			goto unlock;
 		}
 
+		ds_count++;
+
 		if (0 == ISSET_VALUE(result))
 			continue;
 
@@ -2149,6 +2198,12 @@ static int	check_vcenter_datastore_latency(AGENT_REQUEST *request, const char *u
 		UNSET_UI64_RESULT(result);
 	}
 
+	if (0 == ds_count)
+	{
+		SET_MSG_RESULT(result, zbx_strdup(NULL, "No datastores available."));
+		goto unlock;
+	}
+
 	if (0 == is_maxlatency && 0 != count)
 		latency = latency / count;
 
@@ -2165,14 +2220,14 @@ int	check_vcenter_datastore_read(AGENT_REQUEST *request, const char *username, c
 		AGENT_RESULT *result)
 {
 	return check_vcenter_datastore_latency(request, username, password, "datastore/totalReadLatency[average]",
-			result);
+			ZBX_VMWARE_DS_READ_FILTER, result);
 }
 
 int	check_vcenter_datastore_write(AGENT_REQUEST *request, const char *username, const char *password,
 		AGENT_RESULT *result)
 {
 	return check_vcenter_datastore_latency(request, username, password, "datastore/totalWriteLatency[average]",
-			result);
+			ZBX_VMWARE_DS_WRITE_FILTER, result);
 }
 
 int	check_vcenter_vm_cpu_num(AGENT_REQUEST *request, const char *username, const char *password,
@@ -3267,4 +3322,54 @@ out:
 	return ret;
 }
 
+int	check_vcenter_dc_discovery(AGENT_REQUEST *request, const char *username, const char *password,
+		AGENT_RESULT *result)
+{
+	char			*url;
+	zbx_vmware_service_t	*service;
+	struct zbx_json		json_data;
+	int			i, ret = SYSINFO_RET_FAIL;
+
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
+
+	if (1 != request->nparam)
+	{
+		SET_MSG_RESULT(result, zbx_strdup(NULL, "Invalid number of parameters."));
+		goto out;
+	}
+
+	url = get_rparam(request, 0);
+
+	zbx_vmware_lock();
+
+	if (NULL == (service = get_vmware_service(url, username, password, result, &ret)))
+		goto unlock;
+
+	zbx_json_initarray(&json_data, ZBX_JSON_STAT_BUF_LEN);
+
+	for (i = 0; i < service->data->datacenters.values_num; i++)
+	{
+		zbx_vmware_datacenter_t	*datacenter = service->data->datacenters.values[i];
+
+		zbx_json_addobject(&json_data, NULL);
+		zbx_json_addstring(&json_data, "{#DATACENTER}", datacenter->name, ZBX_JSON_TYPE_STRING);
+		zbx_json_addstring(&json_data, "{#DATACENTERID}", datacenter->id, ZBX_JSON_TYPE_STRING);
+		zbx_json_close(&json_data);
+	}
+
+	zbx_json_close(&json_data);
+
+	SET_STR_RESULT(result, zbx_strdup(NULL, json_data.buffer));
+
+	zbx_json_free(&json_data);
+
+	ret = SYSINFO_RET_OK;
+unlock:
+	zbx_vmware_unlock();
+out:
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __func__, zbx_sysinfo_ret_string(ret));
+
+	return ret;
+}
+
 #endif	/* defined(HAVE_LIBXML2) && defined(HAVE_LIBCURL) */
