@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -58,19 +58,26 @@ static int			cached_values;
  *             fields  - [IN]  the definition of data to be packed            *
  *             count   - [IN]  field count                                    *
  *                                                                            *
- * Return value: size of packed data                                          *
+ * Return value: size of packed data or 0 if the message size would exceed    *
+ *               4GB limit                                                    *
  *                                                                            *
  ******************************************************************************/
 static zbx_uint32_t	message_pack_data(zbx_ipc_message_t *message, zbx_packed_field_t *fields, int count)
 {
-	int 		i;
-	zbx_uint32_t	field_size, data_size = 0;
-	unsigned char	*offset = NULL;
+	int 			i;
+	zbx_uint32_t		data_size = 0;
+	zbx_uint64_t		field_size;
+	unsigned char		*offset = NULL;
+	const zbx_uint64_t	max_uint32 = ~(zbx_uint32_t)0;
 
 	if (NULL != message)
 	{
 		/* recursive call to calculate required buffer size */
 		data_size = message_pack_data(NULL, fields, count);
+
+		if (0 == data_size || max_uint32 - message->size < data_size)
+			return 0;
+
 		message->size += data_size;
 		message->data = (unsigned char *)zbx_realloc(message->data, message->size);
 		offset = message->data + (message->size - data_size);
@@ -100,11 +107,15 @@ static zbx_uint32_t	message_pack_data(zbx_ipc_message_t *message, zbx_packed_fie
 			if (PACKED_FIELD_STRING == fields[i].type)
 			{
 				field_size = (NULL != fields[i].value) ? strlen((const char *)fields[i].value) + 1 : 0;
-				fields[i].size = field_size;
+				fields[i].size = (zbx_uint32_t)field_size;
+
 				field_size += sizeof(zbx_uint32_t);
 			}
 
-			data_size += field_size;
+			if (field_size + data_size > max_uint32)
+				return 0;
+
+			data_size += (zbx_uint32_t)field_size;
 		}
 	}
 
@@ -125,13 +136,14 @@ static zbx_uint32_t	message_pack_data(zbx_ipc_message_t *message, zbx_packed_fie
  ******************************************************************************/
 static zbx_uint32_t	preprocessor_pack_value(zbx_ipc_message_t *message, zbx_preproc_item_value_t *value)
 {
-	zbx_packed_field_t	fields[23], *offset = fields;	/* 23 - max field count */
+	zbx_packed_field_t	fields[24], *offset = fields;	/* 24 - max field count */
 	unsigned char		ts_marker, result_marker, log_marker;
 
 	ts_marker = (NULL != value->ts);
-	result_marker = (NULL != value->result);
+	result_marker = (NULL != value->result_ptr->result);
 
 	*offset++ = PACKED_FIELD(&value->itemid, sizeof(zbx_uint64_t));
+	*offset++ = PACKED_FIELD(&value->hostid, sizeof(zbx_uint64_t));
 	*offset++ = PACKED_FIELD(&value->item_value_type, sizeof(unsigned char));
 	*offset++ = PACKED_FIELD(&value->item_flags, sizeof(unsigned char));
 	*offset++ = PACKED_FIELD(&value->state, sizeof(unsigned char));
@@ -146,27 +158,27 @@ static zbx_uint32_t	preprocessor_pack_value(zbx_ipc_message_t *message, zbx_prep
 
 	*offset++ = PACKED_FIELD(&result_marker, sizeof(unsigned char));
 
-	if (NULL != value->result)
+	if (NULL != value->result_ptr->result)
 	{
 
-		*offset++ = PACKED_FIELD(&value->result->lastlogsize, sizeof(zbx_uint64_t));
-		*offset++ = PACKED_FIELD(&value->result->ui64, sizeof(zbx_uint64_t));
-		*offset++ = PACKED_FIELD(&value->result->dbl, sizeof(double));
-		*offset++ = PACKED_FIELD(value->result->str, 0);
-		*offset++ = PACKED_FIELD(value->result->text, 0);
-		*offset++ = PACKED_FIELD(value->result->msg, 0);
-		*offset++ = PACKED_FIELD(&value->result->type, sizeof(int));
-		*offset++ = PACKED_FIELD(&value->result->mtime, sizeof(int));
+		*offset++ = PACKED_FIELD(&value->result_ptr->result->lastlogsize, sizeof(zbx_uint64_t));
+		*offset++ = PACKED_FIELD(&value->result_ptr->result->ui64, sizeof(zbx_uint64_t));
+		*offset++ = PACKED_FIELD(&value->result_ptr->result->dbl, sizeof(double));
+		*offset++ = PACKED_FIELD(value->result_ptr->result->str, 0);
+		*offset++ = PACKED_FIELD(value->result_ptr->result->text, 0);
+		*offset++ = PACKED_FIELD(value->result_ptr->result->msg, 0);
+		*offset++ = PACKED_FIELD(&value->result_ptr->result->type, sizeof(int));
+		*offset++ = PACKED_FIELD(&value->result_ptr->result->mtime, sizeof(int));
 
-		log_marker = (NULL != value->result->log);
+		log_marker = (NULL != value->result_ptr->result->log);
 		*offset++ = PACKED_FIELD(&log_marker, sizeof(unsigned char));
-		if (NULL != value->result->log)
+		if (NULL != value->result_ptr->result->log)
 		{
-			*offset++ = PACKED_FIELD(value->result->log->value, 0);
-			*offset++ = PACKED_FIELD(value->result->log->source, 0);
-			*offset++ = PACKED_FIELD(&value->result->log->timestamp, sizeof(int));
-			*offset++ = PACKED_FIELD(&value->result->log->severity, sizeof(int));
-			*offset++ = PACKED_FIELD(&value->result->log->logeventid, sizeof(int));
+			*offset++ = PACKED_FIELD(value->result_ptr->result->log->value, 0);
+			*offset++ = PACKED_FIELD(value->result_ptr->result->log->source, 0);
+			*offset++ = PACKED_FIELD(&value->result_ptr->result->log->timestamp, sizeof(int));
+			*offset++ = PACKED_FIELD(&value->result_ptr->result->log->severity, sizeof(int));
+			*offset++ = PACKED_FIELD(&value->result_ptr->result->log->logeventid, sizeof(int));
 		}
 	}
 
@@ -597,6 +609,115 @@ zbx_uint32_t	zbx_preprocessor_pack_test_result(unsigned char **data, const zbx_p
 	return size;
 }
 
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_preprocessor_pack_diag_stats                                 *
+ *                                                                            *
+ * Purpose: pack diagnostic statistics data into a single buffer that can be  *
+ *          used in IPC                                                       *
+ * Parameters: data       - [OUT] memory buffer for packed data               *
+ *             total      - [IN] the number of values                         *
+ *             queued     - [IN] the number of values waiting to be           *
+ *                               preprocessed                                 *
+ *             processing - [IN] the number of values being preprocessed      *
+ *             done       - [IN] the number of values waiting to be flushed   *
+ *                               that are either preprocessed or did not      *
+ *                               require preprocessing                        *
+ *             pending    - [IN] the number of values pending to be           *
+ *                               preprocessed after previous value for        *
+ *                               example delta, throttling depends on         *
+ *                               previous value                               *
+ *             data       - [IN] IPC data buffer                              *
+ *                                                                            *
+ ******************************************************************************/
+zbx_uint32_t	zbx_preprocessor_pack_diag_stats(unsigned char **data, int total, int queued, int processing, int done,
+		int pending)
+{
+	unsigned char	*ptr;
+	zbx_uint32_t	data_len = 0;
+
+	zbx_serialize_prepare_value(data_len, total);
+	zbx_serialize_prepare_value(data_len, queued);
+	zbx_serialize_prepare_value(data_len, processing);
+	zbx_serialize_prepare_value(data_len, done);
+	zbx_serialize_prepare_value(data_len, pending);
+
+	*data = (unsigned char *)zbx_malloc(NULL, data_len);
+
+	ptr = *data;
+	ptr += zbx_serialize_value(ptr, total);
+	ptr += zbx_serialize_value(ptr, queued);
+	ptr += zbx_serialize_value(ptr, processing);
+	ptr += zbx_serialize_value(ptr, done);
+	(void)zbx_serialize_value(ptr, pending);
+
+	return data_len;
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_preprocessor_pack_top_request                                *
+ *                                                                            *
+ * Purpose: pack top request data into a single buffer that can be used in IPC*
+ *                                                                            *
+ * Parameters: data  - [OUT] memory buffer for packed data                    *
+ *             field - [IN] the sort field                                    *
+ *             limit - [IN] the number of top values to return                *
+ *                                                                            *
+ ******************************************************************************/
+zbx_uint32_t	zbx_preprocessor_pack_top_items_request(unsigned char **data, int limit)
+{
+	zbx_uint32_t	data_len = 0;
+
+	zbx_serialize_prepare_value(data_len, limit);
+	*data = (unsigned char *)zbx_malloc(NULL, data_len);
+	(void)zbx_serialize_value(*data, limit);
+
+	return data_len;
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_preprocessor_pack_top_result                                 *
+ *                                                                            *
+ * Purpose: pack top result data into a single buffer that can be used in IPC *
+ *                                                                            *
+ * Parameters: data      - [OUT] memory buffer for packed data                *
+ *             items     - [IN] the array of item references                  *
+ *             items_num - [IN] the number of items                           *
+ *                                                                            *
+ ******************************************************************************/
+zbx_uint32_t	zbx_preprocessor_pack_top_items_result(unsigned char **data, zbx_preproc_item_stats_t **items,
+		int items_num)
+{
+	unsigned char	*ptr;
+	zbx_uint32_t	data_len = 0, item_len = 0;
+	int		i;
+
+	if (0 != items_num)
+	{
+		zbx_serialize_prepare_value(item_len, items[0]->itemid);
+		zbx_serialize_prepare_value(item_len, items[0]->values_num);
+		zbx_serialize_prepare_value(item_len, items[0]->steps_num);
+	}
+
+	zbx_serialize_prepare_value(data_len, items_num);
+	data_len += item_len * items_num;
+	*data = (unsigned char *)zbx_malloc(NULL, data_len);
+
+	ptr = *data;
+	ptr += zbx_serialize_value(ptr, items_num);
+
+	for (i = 0; i < items_num; i++)
+	{
+		ptr += zbx_serialize_value(ptr, items[i]->itemid);
+		ptr += zbx_serialize_value(ptr, items[i]->values_num);
+		ptr += zbx_serialize_value(ptr, items[i]->steps_num);
+	}
+
+	return data_len;
+}
+
 /******************************************************************************
  *                                                                            *
  * Function: zbx_preprocessor_unpack_value                                    *
@@ -618,6 +739,7 @@ zbx_uint32_t	zbx_preprocessor_unpack_value(zbx_preproc_item_value_t *value, unsi
 	unsigned char	*offset = data, ts_marker, result_marker, log_marker;
 
 	offset += zbx_deserialize_uint64(offset, &value->itemid);
+	offset += zbx_deserialize_uint64(offset, &value->hostid);
 	offset += zbx_deserialize_char(offset, &value->item_value_type);
 	offset += zbx_deserialize_char(offset, &value->item_flags);
 	offset += zbx_deserialize_char(offset, &value->state);
@@ -663,7 +785,9 @@ zbx_uint32_t	zbx_preprocessor_unpack_value(zbx_preproc_item_value_t *value, unsi
 		agent_result->log = log;
 	}
 
-	value->result = agent_result;
+	value->result_ptr = (zbx_result_ptr_t *)zbx_malloc(NULL, sizeof(zbx_result_ptr_t));
+	value->result_ptr->result = agent_result;
+	value->result_ptr->refcount = 1;
 
 	return offset - data;
 }
@@ -772,6 +896,87 @@ void	zbx_preprocessor_unpack_test_result(zbx_vector_ptr_t *results, zbx_vector_p
 
 	(void)zbx_deserialize_str(offset, error, value_len);
 }
+
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_preprocessor_unpack_diag_stats                               *
+ *                                                                            *
+ * Purpose: unpack preprocessing test data from IPC data buffer               *
+ *                                                                            *
+ * Parameters: total      - [OUT] the number of values                        *
+ *             queued     - [OUT] the number of values waiting to be          *
+ *                                preprocessed                                *
+ *             processing - [OUT] the number of values being preprocessed     *
+ *             done       - [OUT] the number of values waiting to be flushed  *
+ *                                that are either preprocessed or did not     *
+ *                                require preprocessing                       *
+ *             pending    - [OUT] the number of values pending to be          *
+ *                                preprocessed after previous value for       *
+ *                                example delta, throttling depends on        *
+ *                                previous value                              *
+ *             data       - [IN] IPC data buffer                              *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_preprocessor_unpack_diag_stats(int *total, int *queued, int *processing, int *done,
+		int *pending, const unsigned char *data)
+{
+	const unsigned char	*offset = data;
+
+	offset += zbx_deserialize_int(offset, total);
+	offset += zbx_deserialize_int(offset, queued);
+	offset += zbx_deserialize_int(offset, processing);
+	offset += zbx_deserialize_int(offset, done);
+	(void)zbx_deserialize_int(offset, pending);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_preprocessor_unpack_top_request                              *
+ *                                                                            *
+ * Purpose: unpack preprocessing test data from IPC data buffer               *
+ *                                                                            *
+ * Parameters: data  - [OUT] memory buffer for packed data                    *
+ *             limit - [IN] the number of top values to return                *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_preprocessor_unpack_top_request(int *limit, const unsigned char *data)
+{
+	(void)zbx_deserialize_value(data, limit);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_preprocessor_unpack_top_request                              *
+ *                                                                            *
+ * Purpose: unpack preprocessing test data from IPC data buffer               *
+ *                                                                            *
+ * Parameters: items - [OUT] the item diag data                               *
+ *             data  - [IN] memory buffer for packed data                     *
+ *                                                                            *
+ ******************************************************************************/
+void	zbx_preprocessor_unpack_top_result(zbx_vector_ptr_t *items, const unsigned char *data)
+{
+	int	i, items_num;
+
+	data += zbx_deserialize_value(data, &items_num);
+
+	if (0 != items_num)
+	{
+		zbx_vector_ptr_reserve(items, items_num);
+
+		for (i = 0; i < items_num; i++)
+		{
+			zbx_preproc_item_stats_t	*item;
+
+			item = (zbx_preproc_item_stats_t *)zbx_malloc(NULL, sizeof(zbx_preproc_item_stats_t));
+			data += zbx_deserialize_value(data, &item->itemid);
+			data += zbx_deserialize_value(data, &item->values_num);
+			data += zbx_deserialize_value(data, &item->steps_num);
+			zbx_vector_ptr_append(items, item);
+		}
+	}
+}
+
 /******************************************************************************
  *                                                                            *
  * Function: preprocessor_send                                                *
@@ -816,9 +1021,10 @@ static void	preprocessor_send(zbx_uint32_t code, unsigned char *data, zbx_uint32
  *                                                                            *
  * Function: zbx_preprocess_item_value                                        *
  *                                                                            *
- * Purpose: perform item value preprocessing and dependend item processing    *
+ * Purpose: perform item value preprocessing and dependent item processing    *
  *                                                                            *
  * Parameters: itemid          - [IN] the itemid                              *
+ *             itemid          - [IN] the hostid                              *
  *             item_value_type - [IN] the item value type                     *
  *             item_flags      - [IN] the item flags (e. g. lld rule)         *
  *             result          - [IN] agent result containing the value       *
@@ -829,15 +1035,48 @@ static void	preprocessor_send(zbx_uint32_t code, unsigned char *data, zbx_uint32
  *                               ITEM_STATE_NOTSUPPORTED                      *
  *                                                                            *
  ******************************************************************************/
-void	zbx_preprocess_item_value(zbx_uint64_t itemid, unsigned char item_value_type, unsigned char item_flags,
-		AGENT_RESULT *result, zbx_timespec_t *ts, unsigned char state, char *error)
+void	zbx_preprocess_item_value(zbx_uint64_t itemid, zbx_uint64_t hostid, unsigned char item_value_type,
+		unsigned char item_flags, AGENT_RESULT *result, zbx_timespec_t *ts, unsigned char state, char *error)
 {
-	zbx_preproc_item_value_t	value = {.itemid = itemid, .item_value_type = item_value_type, .result = result,
+	zbx_preproc_item_value_t	value = {.itemid = itemid, .hostid = hostid, .item_value_type = item_value_type,
 					.error = error, .item_flags = item_flags, .state = state, .ts = ts};
+	zbx_result_ptr_t		result_ptr = {.result = result};
+	size_t				value_len = 0, len;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
-	preprocessor_pack_value(&cached_message, &value);
+	if (ITEM_STATE_NORMAL == state)
+	{
+		if (0 != ISSET_STR(result))
+			value_len = strlen(result->str);
+
+		if (0 != ISSET_TEXT(result))
+		{
+			if (value_len < (len = strlen(result->text)))
+				value_len = len;
+		}
+
+		if (0 != ISSET_LOG(result))
+		{
+			if (value_len < (len = strlen(result->log->value)))
+				value_len = len;
+		}
+
+		if (ZBX_MAX_RECV_DATA_SIZE < value_len)
+		{
+			result_ptr.result = NULL;
+			value.state = ITEM_STATE_NOTSUPPORTED;
+			value.error = "Value is too large.";
+		}
+	}
+
+	value.result_ptr = &result_ptr;
+
+	if (0 == preprocessor_pack_value(&cached_message, &value))
+	{
+		zbx_preprocessor_flush();
+		preprocessor_pack_value(&cached_message, &value);
+	}
 
 	if (MAX_VALUES_LOCAL < ++cached_values)
 		zbx_preprocessor_flush();
@@ -1025,3 +1264,79 @@ out:
 	return ret;
 }
 
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_preprocessor_get_diag_stats                                  *
+ *                                                                            *
+ * Purpose: get preprocessing manager diagnostic statistics                   *
+ *                                                                            *
+ ******************************************************************************/
+int	zbx_preprocessor_get_diag_stats(int *total, int *queued, int *processing, int *done,
+		int *pending, char **error)
+{
+	unsigned char	*result;
+
+	if (SUCCEED != zbx_ipc_async_exchange(ZBX_IPC_SERVICE_PREPROCESSING, ZBX_IPC_PREPROCESSOR_DIAG_STATS,
+			SEC_PER_MIN, NULL, 0, &result, error))
+	{
+		return FAIL;
+	}
+
+	zbx_preprocessor_unpack_diag_stats(total, queued, processing, done, pending, result);
+	zbx_free(result);
+
+	return SUCCEED;
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_preprocessor_get_top_items                                   *
+ *                                                                            *
+ * Purpose: get the top N items by the number of queued values                *
+ *                                                                            *
+ ******************************************************************************/
+static int	preprocessor_get_top_items(int limit, zbx_vector_ptr_t *items, char **error, zbx_uint32_t code)
+{
+	int		ret;
+	unsigned char	*data, *result;
+	zbx_uint32_t	data_len;
+
+	data_len = zbx_preprocessor_pack_top_items_request(&data, limit);
+
+	if (SUCCEED != (ret = zbx_ipc_async_exchange(ZBX_IPC_SERVICE_PREPROCESSING, code, SEC_PER_MIN, data, data_len,
+			&result, error)))
+	{
+		goto out;
+	}
+
+	zbx_preprocessor_unpack_top_result(items, result);
+	zbx_free(result);
+out:
+	zbx_free(data);
+
+	return ret;
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_preprocessor_get_top_items                                   *
+ *                                                                            *
+ * Purpose: get the top N items by the number of queued values                *
+ *                                                                            *
+ ******************************************************************************/
+int	zbx_preprocessor_get_top_items(int limit, zbx_vector_ptr_t *items, char **error)
+{
+	return preprocessor_get_top_items(limit, items, error, ZBX_IPC_PREPROCESSOR_TOP_ITEMS);
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_preprocessor_get_top_oldest_preproc_items                    *
+ *                                                                            *
+ * Purpose: get the oldest items with preprocessing still in queue            *
+ *                                                                            *
+ ******************************************************************************/
+int	zbx_preprocessor_get_top_oldest_preproc_items(int limit, zbx_vector_ptr_t *items, char **error)
+{
+	return preprocessor_get_top_items(limit, items, error, ZBX_IPC_PREPROCESSOR_TOP_OLDEST_PREPROC_ITEMS);
+}
